//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoSDKCommon
import KakaoSDKAuth

//MARK: - Model

class ApiParams : Codable {
    var name: String
    var paramItems: [ParamItem]
    init(name:String, paramItems:[ParamItem]) {
        self.name = name
        self.paramItems = paramItems
    }
}

class ApiParamsInfo : Codable {
    var info: [ApiKey:ApiParams] = [:]
    init() {
    }
}

enum ParamValue : Codable  {
    case editableText(String)
    case multiSelectableItem([String])
    case singleSelectableItem([String])
}

class ParamItem : Codable {
    var key: String
    var value: ParamValue
    let placeHolderValue: String?
    var selected: Bool
    var selectedIndex:[Int]
    let editable: Bool
    
    init(key: String, value: ParamValue, placeHolderValue: String? = nil, selectedIndex: [Int] = [], selected: Bool = false, editable: Bool = true) {
        self.key = key
        self.value = value
        self.placeHolderValue = placeHolderValue
        self.selectedIndex = selectedIndex
        self.selected = selected
        self.editable = editable
    }
    
    func update(newValue:String? = nil) {
        switch value {
        case .editableText(_):
            if let newValue = newValue {
                value = .editableText(newValue)
            }
        @unknown default:
            break
        }
    }
    
    func select(_ on:Bool? = nil) {
        switch value {
        case .editableText(let currentValue):
            if let on = on {
                selected = on
            }
            else {
                if currentValue.count == 0 { return }
                
                selected = !selected
            }
        @unknown default:
            break
        }
    }
    
    func select(index:Int) -> Bool {
        var selectedAtIndex = false
        switch value {
        case .multiSelectableItem(_):
            if selectedIndex.contains(index) {
                selectedIndex = selectedIndex.filter({ oldIndex in
                    return oldIndex != index
                })
                selectedAtIndex = false
            }
            else {
                selectedIndex.append(index)
                selectedAtIndex = true
            }
            (selectedIndex.count > 0) ? (selected = true) : (selected = false)
        case .singleSelectableItem(_):
            if selectedIndex.contains(index) {
                selectedIndex.removeAll()
                selectedAtIndex = false
            }
            else {
                selectedIndex.removeAll()
                selectedIndex.append(index)
                selectedAtIndex = true
            }
            (selectedIndex.count > 0) ? (selected = true) : (selected = false)
        @unknown default:
            break
        }
        return selectedAtIndex
    }
    
    func isSelectedAt(index:Int) -> Bool {
        var isSelected = false
        
        switch value {
        case .multiSelectableItem(_):
            fallthrough
        case .singleSelectableItem(_):
            if selectedIndex.contains(index) {
                isSelected = true
            }
        @unknown default:
            break
        }
        return isSelected
    }
    
    func isSelected() -> Bool {
        var isSelected = false
        switch value {
        case .editableText(_):
            if selected == true { isSelected = true }
        case .multiSelectableItem(_):
            fallthrough
        case .singleSelectableItem(_):
            if (selected == true && selectedIndex.count > 0) {
                isSelected = true
            }
        @unknown default:
            break
        }
        return isSelected
    }
    
    func resetSelect() {
        switch value {
        case .editableText(_):
            selected = false
        case .multiSelectableItem(_):
            fallthrough
        case .singleSelectableItem(_):
            selectedIndex.removeAll()
            selected = false            
        @unknown default:
            break
        }
        
    }
    
}

// MARK: - codable helper
enum AnyValue : Codable {
    case string(String)
    case int(Int)
    case bool(Bool)
    
    init(from decoder : Decoder) throws {
        let container = try decoder.singleValueContainer()
        if let int = try? container.decode(Int.self) {
            self = .int(int)
            return
        }
        if let string = try? container.decode(String.self) {
            self = .string(string)
            return
        }
        if let bool = try? container.decode(Bool.self) {
            self = .bool(bool)
            return
        }
        throw AnyValueError.missingValue
    }
    
    func encode(to encoder: Encoder) throws {
        var container = encoder.singleValueContainer()
        switch self {
        case .string(let value):
            try container.encode(value)
        case .int(let value):
            try container.encode(value)
        case .bool(let value):
            try container.encode(value)
        }
    }
    
    enum AnyValueError:Error {
            case missingValue
        }
}
